﻿SET client_encoding = 'UTF8';

ALTER TABLE public.kn
  ALTER COLUMN other TYPE TEXT COLLATE pg_catalog."default";


CREATE OR REPLACE FUNCTION public.ol_toch_ins_after (
)
RETURNS trigger AS
$body$
DECLARE
  idu INTEGER;
  idmsk VARCHAR;
  ngeom geometry;
  idt INTEGER;
  pgeom geometry;
  n INTEGER;
  p CHAR;
  sposob CHAR;
BEGIN
  idt = NEW.id;
  idu = NEW.id_uchastok;
  ngeom = NEW.geom;

  INSERT INTO public.granica_tochka (id_tochka, id_granica) 
    SELECT idt, id FROM msk.granica WHERE id_uchastok=idu
      AND public.st_intersects(ngeom, geom);
   
-- если уже существует точка с такой геометрией, тогда номер и префикс берётся от существующей
-- если образование ЗУ из земель госсобственности - тогда все точки новые
-- если раздел ЗУ - тогда все совпадающие новые точки с одинаковым номером
  IF (idt IS NOT NULL) THEN
    pgeom = NEW.geom;
    SELECT nomer, prefiks_nomera FROM msk.tochka 
        WHERE (id <> idt) 
            AND pre = 0
            AND (public.st_equals(pgeom, geom)) 
        INTO n, p;
    
    IF (n > 0) THEN
      SELECT id_sposob_obrazovaniya FROM msk.uchastok WHERE id = idu INTO sposob;
      
      IF (p = 'н') THEN
        IF (sposob IN ('2', '3', '4')) THEN
          UPDATE msk.tochka SET (nomer, prefiks_nomera) = (n, p) Where id = idt;
        END IF;    
      
      ELSE
        IF (sposob IS NOT NULL) AND (sposob <> '5') THEN
          UPDATE msk.tochka SET (nomer, prefiks_nomera) = (n, p) Where id = idt;
          
        END IF;    
      END IF;
    END IF; 
    
-- если точка новая, границы с её участием становятся красными    
    SELECT prefiks_nomera FROM msk.tochka WHERE id = idt INTO p;
    IF p = 'н' THEN
      UPDATE msk.granica SET uslovnyy_znak = 1 WHERE id IN
        (SELECT id_granica FROM public.granica_tochka WHERE id_tochka = idt);
    END IF;    
     
  END IF;  

-- добавим новую точку в таблицу уникальных точек
  BEGIN
    INSERT INTO msk.tochka_uni 
      SELECT id, id_uchastok, nomer, prefiks_nomera, geom, 
             podpis_x, podpis_y, uslovnyy_znak, id_msk, pre
        FROM msk.tochka
        WHERE id = idt;
  
  EXCEPTION WHEN OTHERS THEN
    -- nothing
  END;          
  
  RETURN NEW;
END;
$body$
LANGUAGE 'plpgsql'
VOLATILE
CALLED ON NULL INPUT
SECURITY INVOKER
COST 100;


CREATE OR REPLACE FUNCTION public.ol_uch_upd (
)
RETURNS trigger AS
$body$
DECLARE
  iduch INTEGER;
  idkat CHAR(12);
  label CHAR(50);
  idsposob CHAR(1);
  nomk SMALLINT;
  geomUch geometry;
BEGIN
  iduch = NEW.id;

  -- если меняется категория земель у многоконтурного, 
  -- значит меняется категория и у всех контуров
  IF ((OLD.id_kategoriya IS NULL) OR (NEW.id_kategoriya <> OLD.id_kategoriya)) 
    AND (NEW.id_vid_uchastka = '05') THEN
    
    idkat = NEW.id_kategoriya;
    UPDATE msk.uchastok SET id_kategoriya = idkat 
      WHERE id IN (SELECT id_children 
                    FROM public.parcel_parcel 
                    WHERE id_parent = iduch);   
  END IF;

  -- если меняется обозначение на плане у многоконтурного, 
  -- значит меняется обозначение на плане и у всех контуров
  IF ((OLD.oboznachenie_na_plane IS NULL) 
        OR (NEW.oboznachenie_na_plane <> OLD.oboznachenie_na_plane)) 
    AND (TRIM(NEW.oboznachenie_na_plane) > ' ') THEN
    
    label = TRIM(NEW.oboznachenie_na_plane);
    UPDATE msk.uchastok 
      SET oboznachenie_na_plane = (label||'('||nomer_kontura||')')
      WHERE id IN (SELECT id_children 
                    FROM public.parcel_parcel 
                    WHERE id_parent = iduch);   
  END IF;

  -- если меняется способ образования(нумерации новых точек) земель у основного ЗУ, 
  -- значит меняется и у всех контуров/входящих
  IF ((OLD.id_sposob_obrazovaniya IS NULL) 
        OR (NEW.id_sposob_obrazovaniya <> OLD.id_sposob_obrazovaniya)) 
    AND (NEW.id_vid_uchastka IN ('05', '02')) THEN

    idsposob = NEW.id_sposob_obrazovaniya;
    UPDATE msk.uchastok SET id_sposob_obrazovaniya = idsposob 
      WHERE id IN (SELECT id_children 
                    FROM public.parcel_parcel 
                    WHERE id_parent = iduch);   
  END IF;

  -- если меняется геометрия ЗУ, происходит проверка на наличие точек, 
  -- прекращающих существование
  IF NOT st_equals(NEW.geom, OLD.geom) THEN
    geomUch = NEW.geom;
    UPDATE msk.tochka SET pre = 1, poryadok_obhoda = 0
      WHERE id_uchastok = iduch AND 
        (st_disjoint(geom, geomUch) OR st_within(geom, geomUch)); 
  END IF;  
  
  NEW.create_user = OLD.create_user;
  NEW.create_date = OLD.create_date;
  NEW.update_user = current_user;
  NEW.update_date = current_timestamp;

  RETURN NEW;
END;
$body$
LANGUAGE 'plpgsql'
VOLATILE
CALLED ON NULL INPUT
SECURITY INVOKER
COST 100;

UPDATE class.database_version SET (number_version, date_version) = (20, '2014-06-27');
